"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var scriptable_banner_exports = {};
__export(scriptable_banner_exports, {
  scriptableBanner: () => scriptableBanner
});
module.exports = __toCommonJS(scriptable_banner_exports);
var import_generateScriptableBanner = require("@scriptables/manifest/generateScriptableBanner");
var import_validateManifest = require("@scriptables/manifest/validateManifest");
var import_fs = require("fs");
var import_promises = require("fs/promises");
var import_path = require("path");
var import_logger = require("./logger");
var import_utils = require("./utils");
const DEFAULT_MANIFEST_EXTENSIONS = [".manifest.json", ".manifest", ".json"];
function scriptableBanner(options = {}) {
  const {
    manifestExtensions = DEFAULT_MANIFEST_EXTENSIONS,
    warnOnMissingManifest = true,
    warnOnInvalidManifest = true,
    warnOnMissingEntry = true,
    getManifestPath,
    patterns = ["**/*.{ts,tsx,js,jsx}"]
  } = options;
  const logger = (0, import_logger.createLogger)("scriptable-banner");
  return {
    name: "scriptable",
    setup(build) {
      let manifestCache = /* @__PURE__ */ new Map();
      if (!build.initialOptions.metafile) {
        logger.warn(
          'Warning: "metafile" option was not enabled. Automatically setting "metafile: true" to allow scriptable plugin to modify output files.'
        );
        build.initialOptions.metafile = true;
      }
      build.onStart(() => {
        manifestCache.clear();
      });
      build.onResolve({ filter: /.*/ }, async (args) => {
        if (!(0, import_utils.isMatch)(args.path, patterns)) {
          return null;
        }
        if (Array.isArray(build.initialOptions.entryPoints) && build.initialOptions.entryPoints.map(String).includes(args.path)) {
          const dir = (0, import_path.dirname)(args.path);
          const baseFileName = (0, import_path.basename)(args.path).replace(/\.[tj]sx?$/, "");
          const manifestPath = getManifestPath ? getManifestPath(args.path) : manifestExtensions.map((ext) => (0, import_path.resolve)(dir, `${baseFileName}${ext}`)).find(import_fs.existsSync);
          if (manifestPath) {
            try {
              const manifestContent = await (0, import_promises.readFile)(manifestPath, "utf8");
              const manifest = JSON.parse(manifestContent);
              const normalizedPath = (0, import_utils.normalizePath)(args.path);
              try {
                (0, import_validateManifest.validateManifest)(manifest);
              } catch (error) {
                if (warnOnInvalidManifest && error instanceof Error) {
                  logger.warn(`Invalid manifest file: ${manifestPath} - ${error.message}`, error);
                }
                return null;
              }
              manifestCache.set(normalizedPath, manifest);
            } catch (error) {
              logger.warn(`Failed to read manifest file: ${manifestPath}`, error);
            }
          } else if (warnOnMissingManifest) {
            logger.warn(`No manifest file found for entry point: ${args.path}`);
          }
        }
        return null;
      });
      build.onEnd(async (result) => {
        if (!result.metafile || Object.keys(result.metafile.outputs).length === 0) {
          logger.warn(
            'Skipping banner addition because "metafile" is not available or contains no outputs. Ensure "metafile: true" is set in esbuild options if write is enabled.'
          );
          return;
        }
        const outputs = result.metafile.outputs;
        for (const outputPath of Object.keys(outputs)) {
          const outputMeta = outputs[outputPath];
          const entryPoint = outputMeta?.entryPoint ?? "";
          const normalizedEntryPath = entryPoint ? import_path.posix.normalize(entryPoint.split(import_path.sep).join("/")) : "";
          if (!normalizedEntryPath || !manifestCache.has(normalizedEntryPath)) {
            if (warnOnMissingEntry) {
              logger.warn(`No matching entry point found for output file: ${outputPath}`);
            }
            continue;
          }
          try {
            const manifest = manifestCache.get(normalizedEntryPath) || {};
            const banner = (0, import_generateScriptableBanner.generateScriptableBanner)(manifest);
            if (result.outputFiles) {
              const outputFile = result.outputFiles.find((file) => file.path === outputPath || file.path === "<stdout>");
              if (outputFile) {
                outputFile.contents = Buffer.from(banner + outputFile.text.trimEnd());
              }
            } else {
              try {
                await (0, import_utils.waitForFile)(outputPath);
                const content = await (0, import_promises.readFile)(outputPath, "utf-8");
                const updatedContent = banner + content.trimEnd();
                await (0, import_promises.writeFile)(outputPath, updatedContent, "utf8");
              } catch (error) {
                logger.error(
                  `Failed to process file ${outputPath}: ${error instanceof Error ? error.message : String(error)}`
                );
              }
            }
          } catch (error) {
            logger.error(`Failed to process output file: ${outputPath}`, error);
          }
        }
      });
    }
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  scriptableBanner
});
