"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var scriptable_deploy_exports = {};
__export(scriptable_deploy_exports, {
  scriptableDeploy: () => scriptableDeploy
});
module.exports = __toCommonJS(scriptable_deploy_exports);
var import_generateScriptableBanner = require("@scriptables/manifest/generateScriptableBanner");
var import_findicloud = require("findicloud");
var import_promises = require("fs/promises");
var import_path = require("path");
var import_logger = require("./logger");
var import_utils = require("./utils");
const findICloudBugsUrl = "https://github.com/hexxspark/findicloud/issues";
async function tryReadManifest(filePath, extensions) {
  for (const ext of extensions) {
    try {
      const manifestPath = filePath.replace(/\.[^.]+$/, "") + ext;
      const content = await (0, import_promises.readFile)(manifestPath, "utf-8");
      return JSON.parse(content);
    } catch {
      continue;
    }
  }
  return null;
}
function scriptableDeploy(options = {}) {
  const {
    verbose = false,
    continueOnError = true,
    scriptableDir,
    patterns = ["**/*.{ts,tsx,js,jsx}"],
    addBanner = true,
    manifestExtensions = [".manifest.json", ".manifest", ".json"]
  } = options;
  const filterFile = (0, import_utils.createPatternMatcher)(patterns);
  const logger = (0, import_logger.createLogger)("scriptable-deploy", {
    level: verbose ? "verbose" : "info"
  });
  let targetDir;
  return {
    name: "scriptable-deploy",
    setup(build) {
      if (!build.initialOptions.metafile) {
        logger.verbose("Automatically enabling metafile option");
        build.initialOptions.metafile = true;
      }
      build.onStart(async () => {
        try {
          if (scriptableDir) {
            targetDir = scriptableDir;
          } else {
            const [found] = await (0, import_findicloud.findICloudDrivePaths)({ appNamePattern: "Scriptable", types: [import_findicloud.PathType.APP_STORAGE] });
            targetDir = found?.path;
          }
          if (!targetDir) {
            throw new Error(
              `Scriptable directory not found in iCloud Drive. Please ensure iCloud Drive is installed and Scriptable app is synced with iCloud. If iCloud Drive is not installed, you can download it from: https://support.apple.com/en-us/HT204283. If the issue persists, please report it at: ${findICloudBugsUrl}`
            );
          }
          logger.verbose(`Target directory: ${targetDir}`);
        } catch (error) {
          const message = `Failed to find Scriptable directory: ${error.message}`;
          if (!continueOnError) {
            throw new Error(message);
          }
          logger.error(message);
        }
      });
      build.onEnd(async (result) => {
        if (!result.metafile || !targetDir) {
          logger.error("Missing required configuration");
          return;
        }
        try {
          const outputs = Object.keys(result.metafile.outputs).filter((file) => {
            const { entryPoint } = result.metafile.outputs[file];
            return entryPoint && filterFile(entryPoint);
          });
          for (const file of outputs) {
            const output = result.metafile.outputs[file];
            let content;
            if (result.outputFiles) {
              const outputFile = result.outputFiles.find((f) => (0, import_utils.isPathEqual)(f.path, file));
              if (!outputFile) {
                throw new Error(`Output file not found: ${file}`);
              }
              content = new TextDecoder().decode(outputFile.contents);
            } else {
              await (0, import_utils.waitForFile)(file);
              content = await (0, import_promises.readFile)(file, "utf-8");
            }
            let targetFileName = (0, import_path.basename)(file);
            let manifest = null;
            try {
              const inputs = Object.entries(output.inputs);
              if (!inputs.length) {
                throw new Error(`No input files found for ${file}`);
              }
              const sourceFile = inputs[0][0];
              if (!sourceFile) {
                throw new Error(`Invalid source file for ${file}`);
              }
              manifest = await tryReadManifest(sourceFile, manifestExtensions);
              if (manifest?.name) {
                targetFileName = `${manifest.name}.js`;
              }
            } catch (error) {
              logger.warn(`Failed to read manifest for ${file}: ${error}`);
            }
            const targetPath = (0, import_path.join)(targetDir, targetFileName);
            if (addBanner && manifest) {
              try {
                const banner = (0, import_generateScriptableBanner.generateScriptableBanner)(manifest);
                const newContent = banner + "\n" + content;
                await (0, import_promises.writeFile)(targetPath, newContent);
                logger.info(`Deployed with banner: ${file} -> ${targetPath}`);
                continue;
              } catch (error) {
                logger.warn(`Failed to add banner to ${file}: ${error}`);
              }
            }
            await (0, import_promises.writeFile)(targetPath, content);
            logger.info(`Deployed: ${file} -> ${targetPath}`);
          }
          logger.info(`Successfully deployed ${outputs.length} files`);
        } catch (error) {
          logger.error(`Deploy failed: ${error.message}`);
          if (!continueOnError) {
            throw error;
          }
        }
      });
    }
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  scriptableDeploy
});
